include std/error.e
include std/filesys.e
include std/os.e
include std/text.e
include std/socket.e as sock
include myLibs/myDebug.e
include myLibs/common.e
include myLibs/myFile.e
include myLibs/mySeq.e
--include myLibs/myConv.e
include myLibs/myConsole.e
include tcplib.e

constant AVAILABLE_WIDGETS = {
  "TScrollBar", "TGroupBox", "TComboBox", "TListBox", "TEdit", "TMemo",
  "TStaticText", "TButton", "TCheckBox", "TToggleBox", "TRadioButton", "TLabel" }

integer cnx
sequence widgetList = {"Form1"}

--------------------------------------------------------------------------------

public function lazGUI_connect(sequence remote_addr, integer port=1500, integer rtn_id=0)
  integer cnx = tcpConnect(remote_addr, port, NONE, rtn_id)
  if cnx = 0 then return 0 end if
  object data = tcpReceive(cnx)
  if sequence(data) then puts(1, data) end if
  if match("lazGUI Ready",data) then
    return cnx
  else
    logMsg(sprintf("cnx error, server reply = \"%s\"!\n", {data}))
    return 0
  end if
end function

--------------------------------------------------------------------------------

public function lazGUI_disconnect(integer cnx)
  if activeConnections[cnx][TCP_STATUS] = DISCONNECTED then return 0 end if
  object data = tcpSendReceive(cnx,"quit")
  tcpClose(cnx)
  if match("lazGUI Exiting",data) then
    return data
  else
    logMsg(sprintf("Disconnection error, server reply = \"%s\"!\n", {data}))
    return 0
  end if
end function

--------------------------------------------------------------------------------

function convertParams(sequence params)
  sequence res = ""
  for i = 1 to length(params) do
    res &= params[i][1] & "="
    if sequence(params[i][2]) then
      res &= quote(params[i][2], '"') & ","
    else
      res &= sprintf("%d",params[i][2]) & ","
    end if
  end for
  return res[1..$-1]
end function

--------------------------------------------------------------------------------

public function createWidget(sequence class, sequence params)
  if not find(class, AVAILABLE_WIDGETS) then
    logMsg(class & " not available")
    return 0
  end if
  object data = tcpSendReceive(cnx,
      "create " & class & " {" &
        convertParams(params) &
      "}"
  )
  if atom(data) then
    logMsg(sprintf("Could not create %s widget!\n", {class}))
    return 0
  end if
  return trim_tail(data)
end function

--------------------------------------------------------------------------------

public function destroyWidget(sequence name)
  if not find(name, widgetList) then
    logMsg(name & " unknown")
    return 0
  end if
  object data = tcpSendReceive(cnx,
      "destroy " & name & " {}"
  )
  if atom(data) then
    logMsg(sprintf("Could not destroy %s widget!\n", {name}))
    return 0
  end if
  return trim_tail(data)
end function

--------------------------------------------------------------------------------

public function setProperties(sequence name, sequence params)
  if not find(name, widgetList) then
    logMsg(name & " unknown")
    return 0
  end if
  object data = tcpSendReceive(cnx,
      "set " & name & " {" &
        convertParams(params) &
      "}"
  )
  if atom(data) then
    logMsg(sprintf("Could not modify %s widget properties!\n", {name}))
    return 0
  end if
  return trim_tail(data)
end function

--------------------------------------------------------------------------------

public function getProperties(sequence name, sequence params)
  if not find(name, widgetList) then
    logMsg(name & " unknown")
    return 0
  end if
  object data = tcpSendReceive(cnx,
      "get " & name & " {" &
        params &
      "}"
  )
  if atom(data) then
    logMsg(sprintf("Could not retrieve %s widget properties!\n", {name}))
    return 0
  end if
  return trim_tail(data)
end function

------------------------------------------------------------------------------

procedure manageNotifications(integer cnx, sequence notification)
  sequence host = activeConnections[cnx][TCP_HOST]
  logMsg(sprintf("Got '%s' from %s", {notification, host}))
  if equal(notification, "exit") then activeConnections[cnx][TCP_STATUS] = DISCONNECTED end if 
end procedure

--------------------------------------------------------------------------------

  sequence cmd = command_line()
  sequence info = pathinfo(cmd[2])
  void = chdir(info[PATH_DIR])
  InitialDir = current_dir()
  f_debug = open(InitialDir&SLASH&info[PATH_BASENAME]&".log", "w")
  crash_file(InitialDir&SLASH&info[PATH_BASENAME]&".err")
  debugLevel = 2

  cnx = lazGUI_connect("127.0.0.1", 1500, routine_id("manageNotifications"))
  if not cnx then 
    close(f_debug)
    abort(1)
  end if

  object data = setProperties("Form1", {
        {"Width",800},
        {"Height",600},
        {"Caption","Title Changed"}
      }
  )
  object btn = createWidget("TButton", {
        {"Left",10},
        {"Top",20},
        {"Width",80},
        {"Height",25},
        {"Caption","Button"}
      }
  )
  widgetList = append(widgetList, btn)
  object combo = createWidget("TComboBox", {
        {"Left",10},
        {"Top",50},
        {"Width",120},
        {"Height",25},
        {"Text","ComboBox"},
        {"Items.CommaText","First,Second,Third"},
        {"Visible",1}
      }
  )
  widgetList = append(widgetList, combo)
  object list = createWidget("TListBox", {
        {"Left",10},
        {"Top",80},
        {"Width",120},
        {"Height",65},
        {"Items.CommaText","First,Second,Third"}
      }
  )
  widgetList = append(widgetList, list)
  object memo = createWidget("TMemo", {
        {"Left",10},
        {"Top",160},
        {"Width",120},
        {"Height",65},
        {"Lines.CommaText","First,Second,Third"}
      }
  )
  widgetList = append(widgetList, memo)
  sleep(5)
  data = setProperties(combo, {
        {"Width",160},
        {"Text","Text Changed"}
      }
  )
  sleep(5)
  data = destroyWidget(combo)
  
  data = getProperties("Form1",
    "Width,Height"
  )
  integer key = -1
  while activeConnections[cnx][TCP_STATUS] = CONNECTED do
    checkActiveConnections()
    key = get_key()    
    if key = 'q' then
      logMsg("Exit requested")
      exit
    end if 
  end while

  object ok = lazGUI_disconnect(cnx)
  close(f_debug)
