unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Contnrs, Forms, Controls, Graphics,
  StdCtrls, Dialogs, lNetComponents, lNet;

const
  CrLf = #13+#10;

type

  TNameObject = class(TCollectionItem) // class for the property 'obj' and the TCollection
  private
    fName: String;
  published
    property name: String read fName write fName;
  end;

  TBaseObject = class(TPersistent)  // class for the JSON structure
  private
    fid: Integer;
    fObj: TNameObject;
    fColl: TCollection;
    fStrings: TStrings;
  public
    constructor Create;
    destructor Destroy; override;
  published  // all properties have published
    property id: Integer read fid write fid;
    property obj: TNameObject read fObj write fObj;
    property coll: TCollection read fColl;
    property strings: TStrings read fStrings;
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Server: TLTCPComponent;
    procedure FormActivate(Sender: TObject);
    procedure FormClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ServerAccept(aSocket: TLSocket);
    procedure ServerDisconnect(aSocket: TLSocket);
    procedure ServerError(const msg: string; aSocket: TLSocket);
    procedure ServerReceive(aSocket: TLSocket);
  private
    { private declarations }
    ControlList: TComponentList;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses typinfo, (*IniFiles,*) Common, hood1;

{ TBaseObject }

constructor TBaseObject.Create;
begin
  // Collection and StringList constructor
  fColl    := TCollection.Create(TNameObject);
  fStrings := TStringList.Create;
  fObj     := TNameObject.Create(nil);
end;

destructor TBaseObject.Destroy;
begin
  // Collection and StringList destructor
  fColl.Free;
  fStrings.Free;
  fObj.Free;
  inherited Destroy;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  fName: string;
begin
  InitialDir := ExtractFileDir(ParamStr(0));
  fName := ExtractFileName(ParamStr(0));
  AssignFile(f_debug, fName + '.log');
  RegisterClasses([TScrollBar, TGroupBox, TComboBox, TListBox, TEdit, TMemo, TStaticText, TButton, TCheckBox, TToggleBox, TRadioButton, TLabel]);
  ControlList := TComponentList.Create;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ControlList.Free;
end;

procedure TForm1.ServerAccept(aSocket: TLSocket);
begin
  Server.SendMessage('lazGUI Ready' + CRLF);
  logMsg('Connection to client with IP ' + aSocket.LocalAddress + ' is up');
end;

procedure TForm1.ServerDisconnect(aSocket: TLSocket);
begin
  logMsg('Connection to client is down');
end;

procedure TForm1.ServerError(const msg: string; aSocket: TLSocket);
begin
  logMsg('Server: ' + msg);
end;

//----------------------------------------------------------------------------

procedure splitInstruction(str: string; var l: TStringList);
// splits string str in tokens according to delimiter sep
var
  i: integer;
  s: string;
  inString: boolean;
  inParams: boolean;
begin
  l.Clear;
  s := '';
  inString := false;      // true if between double-quotes
  inParams := false;      // true if between brackets
  for i := 1 to length(str) do begin
    if (str[i] = '"') then begin
      s := s + str[i];
      inString := not inString;
    end else if str[i] = '{' then
      inParams := true
    else if str[i] = '}' then
      inParams := false
    else if (not inString) and (not inParams) and (str[i] = ' ') then begin
      l.Add(s);
      s := '';
    end else
      s := s + str[i];
  end;
  l.Add(s);
end;

function findWinControl(cName: string): integer;
var
  i: integer;
begin
  for i := 0 to Form1.ControlList.Count-1 do begin
    if TWinControl(Form1.ControlList.Items[i]).Name = cName then begin
      Result := i;
      exit;
    end;
  end;
  Result := -1;
end;


//----------------------------------------------------------------------------

procedure TForm1.ServerReceive(aSocket: TLSocket);
var
  Instruction: string;
  sl, prop: TStringList;
  i, lg, nb: Integer;
  command, properties, widget_class, widget_name, pName, pValue: String;
  component: TComponent;
  CRef : TPersistentClass;
  Strings: TStringList;
  PropInfo : PPropInfo;
  PropValue: string;
begin
  aSocket.GetMessage(Instruction);
  Instruction := TrimRight(Instruction);
  logMsg('Received: ' + Instruction);
  if Instruction = 'quit' then begin
    Server.SendMessage('lazGUI Exiting' + CRLF);
    Close;
    exit;
  end;
  sl := TStringList.Create;
  splitInstruction(Instruction, sl);
  if sl.Count <> 3 then begin
    logMsg('Wrong instruction format: ' + sl.CommaText);
    sl.Free;
    exit;
  end;
  command := sl[0];
  properties := sl[2];
  logMsg('command: ' + command);
  Strings := TStringList.Create;
  if command = 'create' then begin
    widget_class := sl[1];
    lg := length(widget_class);
    logMsg('create: ' + widget_class);
    CRef := GetClass(widget_class);
    if CRef<>nil then begin
      Form1.ControlList.Add (TControlClass(CRef).Create(Form1));
      TWinControl(Form1.ControlList[Form1.ControlList.Count-1]).Name := copy(widget_class,2,lg) + IntToStr(Form1.ControlList.Count);
    end else begin
      Server.SendMessage('Unregistered component' + CRLF);
      sl.Free;
      exit;
    end;
    Server.SendMessage(TWinControl(Form1.ControlList[Form1.ControlList.Count-1]).Name + CRLF);
    component := Form1.ControlList[Form1.ControlList.Count-1];

    ListComponentProperties(component, Strings);
    logMsg('Component Properties: ' + Strings.CommaText);

  end else if command = 'set' then begin
    widget_name := sl[1];
    logMsg('set: ' + widget_name);
    // find the control to modify
    if widget_name = 'Form1' then begin
      component := Form1;
    end else begin
      nb := findWinControl(widget_name);
      if nb = -1 then begin
        Server.SendMessage('Error: Widget name not found' + CRLF);
        exit;
      end;
      component := Form1.ControlList[nb];
    end;
    Server.SendMessage('OK' + CRLF);

  end else if command = 'get' then begin
    widget_name := sl[1];
    logMsg('get: ' + widget_name);
    // find the control to modify
    if widget_name = 'Form1' then begin
      component := Form1;
    end else begin
      nb := findWinControl(widget_name);
      if nb = -1 then begin
        Server.SendMessage('Error: Widget name not found' + CRLF);
        exit;
      end;
      component := Form1.ControlList[nb];
    end;

  end else if command = 'destroy' then begin
    widget_name := sl[1];
    logMsg('destroy: ' + widget_name);
    // find the control to destroy
    nb := findWinControl(widget_name);
    if nb = -1 then begin
      Server.SendMessage('Error: Widget name not found' + CRLF);
      exit;
    end;
    component := Form1.ControlList[nb];
    ControlList.Remove(component);
    Server.SendMessage('Widget destroyed' + CRLF);
    sl.Free;
    exit;

  end else begin
    Server.SendMessage('Unsupported command' + CRLF);
    sl.Free;
    exit;

  end;

  logMsg('properties: ' + properties);
  sl.Clear;
  nb := splitString(properties,',',sl);

  if (command = 'create') or (command = 'set') then begin
    prop := TStringList.Create;
    for i := 0 to nb-1 do begin
      logMsg('properties['+IntToStr(i)+']: ' + sl[i]);
      if splitString(sl[i],'=',prop) <> 2 then begin
        logMsg('Wrong property: ' + prop.CommaText);
        prop.free;
        sl.Free;
        exit;
      end;
      pName := prop[0];
      pValue := prop[1];
      if isPublishedProp(component, pName) then begin
        case PropType(component, pName) of
          tkAString, tkSString:
            SetStrProp(component, pName, pValue);
          tkInteger, tkBool:
            SetOrdProp(component, pName, StrToInt(pValue));
        end;
      end else begin
        if splitString(pName,'.',prop) <> 2 then begin
          logMsg('Wrong property: ' + prop.CommaText);
          prop.free;
          sl.Free;
          exit;
        end;
        logMsg('Property: ' + prop.CommaText);
        PropInfo := FindPropInfo(component, prop[0]);
        if Assigned(PropInfo) then begin
(*
          PropValue := GetPropValue(component, PropInfo^.Name);
          logMsg(Format('%s: %s = %s', [PropInfo^.Name,
                                        PropInfo^.PropType^.Name,
                                        PropValue]));
*)
          if prop[1] = 'CommaText' then begin
            Strings.CommaText := pValue;
            SetObjectProp(component, prop[0], Strings);
          end;
        end;

      end;
    end;
    prop.free;

  end else if command = 'get' then begin
    for i := 0 to nb-1 do begin
      logMsg('properties['+IntToStr(i)+']: ' + sl[i]);
      pName := sl[i];
      pValue := '';
      if isPublishedProp(component, pName) then begin
        logMsg('isPublishedProp');
        case PropType(component, pName) of
          tkAString, tkSString:
            pValue := GetStrProp(component, pName);
          tkInteger, tkBool:
            pValue := IntToStr(GetOrdProp(component, pName));
        end;
        sl[i] := pName + '=' + pValue;
        logMsg('sl[i] '+sl[i]);
      end else begin
        logMsg('no Published');
        if splitString(pName,'.',prop) <> 2 then begin
          logMsg('Wrong property: ' + prop.CommaText);
          prop.free;
          sl.Free;
          exit;
        end;
        logMsg('Property: ' + prop.CommaText);
        PropInfo := FindPropInfo(component, prop[0]);
        if Assigned(PropInfo) then begin
          if prop[1] = 'CommaText' then begin
            Strings := TStringList(GetObjectProp(component, prop[0]));
            pValue := Strings.CommaText;
            sl[i] := pName + '=' + pValue;
          end;
        end;
        logMsg('sl[i] '+sl[i]);

      end;
    end;
    Server.SendMessage(sl.CommaText + CRLF);
  end;
  if command = 'create' then begin
    if TWinControl(component).Parent = nil then
      TWinControl(component).Parent := Form1;
    TWinControl(component).OnClick := Form1.OnClick;
  end;
  Strings.free;
  sl.Free;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  Rewrite(f_debug);
  if Server.Listen(Server.Port) then
    logMsg('Server listening');
end;

procedure TForm1.FormClick(Sender: TObject);
begin
  logMsg('Clicked on ' + TComponent(Sender).Name);
  if (Sender is TComboBox) then begin
    Server.SendMessage('click '+TComponent(Sender).Name+' '+TComboBox(Sender).Text);
  end else if (Sender is TListBox) then begin
    Server.SendMessage('click '+TComponent(Sender).Name+' '+IntToStr(TListBox(Sender).ItemIndex));
  end else if (Sender is TMemo) then begin
    Server.SendMessage('click '+TComponent(Sender).Name+' '+TMemo(Sender).SelText);
  end else
    Server.SendMessage('click '+TComponent(Sender).Name);
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
var
  i: integer;
begin
  if Server.Connected then
    Server.SendMessage('exit');
  Server.Disconnect(true);
  logMsg('Server deactivated');
  for i := ControlList.Count-1 downto 1 do begin
    ControlList.Remove(ControlList[i]);
  end;
  CloseFile(f_debug);
end;

end.

