-- common.e
-- library of common reusable functions
-- 2012-09-09
-- v1.12 adapted to Euphoria 4.1

include std/pipeio.e as pipe
include myDebug.e

ifdef TINVERSION then          -- tinEWG
  include tin/tinEWG.ew
elsifdef Win32LibVersion then  -- Win32Lib
  include Win32Lib.ew
elsifdef WXEU_0 then           -- wxEuphoria
  include wxeud.e
elsedef                         -- console
  include std/console.e
end ifdef

------------------------------------------------------------------------------

public function commonField(sequence sht, integer fld, object val, integer start=1)
-- lists records from sheet sht with value val in field fld
  sequence res

  if fld = 0 then return {} end if
  res = {}
  for i = start to length(sht) do
    if equal(sht[i][fld], val) then
      res = append(res, i)
    end if
  end for
  return res
end function

------------------------------------------------------------------------------

public function findRecNumByField(sequence sht, integer fld, object id, integer start=1)
-- find the record in sheet sht which ID (value in first column) is id
-- returns the record number if successful
-- returns 0 (atom) if not
  sequence s
  
  s = commonField(sht, fld, id, start)
--  printf(f_debug, "s = %s\n", {sequenceDump(s)})
  if length(s) then
    return s[1]
  else
    return 0
  end if
end function

------------------------------------------------------------------------------

public function findRecordByField(sequence sht, integer fld, object id, integer start=1)
-- find the record in sheet sht which ID (value in first column) is id
-- returns the record (sequence) if successful
-- returns 0 (atom) if not
  integer i = findRecNumByField(sht, fld, id, start)
  if i > 0 then
    return sht[i]
  else
    return 0
  end if
end function

------------------------------------------------------------------------------

public function selectedFields(sequence sht, integer fld, object val, sequence fields, integer start=1)
-- list selected fields from sheet sht records with value val in field fld
-- ex: selectedFields(tab, 2, "John", {1,3}) will list items having
-- "John" in second column and return a sequence containing columns 1 and 3
--NB is this still useful? commonField returns the whole records
  sequence s, res
  integer l

  if (fld = 0) or (length(fields) = 0) then return {} end if
  s = commonField(sht, fld, val, start)
  l = length(s)
  res = {}
  if l = 1 then
    for i = 1 to length(fields) do
      res = append(res, sht[s[1]][fields[i]])
    end for
    return res
  else
    return {}
  end if
end function

------------------------------------------------------------------------------

public function commonFields(sequence sht, sequence fields, sequence vals, integer start=1)
-- list records from sheet sht with selected values in selected fields
-- ex: commonFields(tab, {1,3}, {5,"John"}) will list items having 5 in first column and "John" in third
  sequence res
  integer found
  
  if (length(fields) = 0) or (length(vals) = 0) or (length(fields) != length(vals)) then return {} end if
  res = {}
  for i = start to length(sht) do
    found = 1
    for j = 1 to length(fields) do
      if compare(sht[i][fields[j]], vals[j]) != 0 then
        found = 0
      end if
    end for
    if found = 1 then  
      res = append(res, i)
    end if
  end for
  return res
end function

------------------------------------------------------------------------------

public function execCommand(sequence cmd)
  sequence s = ""
  object z = pipe:create()
  object p = pipe:exec(cmd, z)
  if atom(p) then
    logMsg(sprintf("Failed to exec() with error %x", pipe:error_no()))
    pipe:kill(p)
	return -1
  end if
  object c = pipe:read(p[pipe:STDOUT], 256)
  while sequence(c) and length(c) do
    s &= c
    if atom(c) then
	  logMsg(sprintf("Failed on read with error %x", pipe:error_no()))
      pipe:kill(p)
	  return -1
    end if
    c = pipe:read(p[pipe:STDOUT], 256)
  end while
  --Close pipes and make sure process is terminated
  pipe:kill(p)
  return s
end function

------------------------------------------------------------------------------

ifdef TINVERSION then          -- tinEWG
elsifdef Win32LibVersion then  -- Win32Lib

public procedure execTask(sequence exc, sequence parms, sequence rep)
  integer exit_code

  exit_code = shellExecuteEx("open",
                     exc,
                     parms,
                     rep,
                     SW_MINIMIZE, -- SW_SHOWNORMAL,  --SW_HIDE,
                     0)
  if exit_code > 32 then
    -- logMsg( sprintf("Tche '%s %s' excute", {exc,parms}) )
  elsif exit_code = SE_ERR_FNF then
    logMsg( sprintf("Echec excution tche '%s %s', fichier non trouv", {exc,parms}) )
  elsif exit_code = SE_ERR_PNF then
    logMsg( sprintf("Echec excution tche '%s %s', chemin non trouv", {exc,parms}) )
  elsif exit_code = SE_ERR_ACCESSDENIED then
    logMsg( sprintf("Echec excution tche '%s %s', accs interdit", {exc,parms}) )
  else
--    logMsg( sprintf("Echec excution tche '%s %s', code d'erreur: %d", {exc,parms, exit_code}) )
    displayMessage(sprintf("Echec excution tche '%s %s', code d'erreur: %d", {exc,parms, exit_code}), ERROR_MSG)
  end if
end procedure

elsifdef WXEU_0 then           -- wxEuphoria
elsedef                         -- console
end ifdef
